function solution=bisection(fcn_handle)
% solution=bisection(fcn_handle)
%
% Applies the function called by 'fcn_handle' on the set of candidate
% solutions and returns the solution it finds
%
% Author: Murat Okatan, PhD.
% Citation: Okatan M, Kocatrk M. Truncation thresholds: a pair of spike detection thresholds computed using truncated probability distributions. 
% Turk J Elec Eng & Comp Sci 2016; Available online: 12.05.2016.
% http://online.journals.tubitak.gov.tr/openAcceptedDocument.htm?fileID=696635&no=172411
% DOI: 10.3906/elk-1603-33 (This number will become active after the manuscript has been selected for inclusion in an issue.)
%
% This work is supported by the Scientific Research Project Fund of Cumhuriyet University under the project number TEKNO-002 
% Copyright (c) 2015 Cumhuriyet University
% Bu alma, Cumhuriyet niversitesi Bilimsel Aratrma Projeleri (CBAP) tarafndan TEKNO-002 proje numaras ile desteklenmitir.
% Telif Hakk (c) 2015 Cumhuriyet niversitesi

global SCS

fcn_name='Bisection';
%Find the cardinality of the set of candidate solutions
N=length(SCS);

%The solution will be found in at most log2(N) steps. Allocate memory for intermediate solutions
ns=max(1, ceil(log2(N)));
announce(sprintf('%s: Finding the first solution. At most %d solutions will be obtained.', fcn_name, ns));
output=fcn_handle(); %Call the relevant function and get the first solution. 
counter=1;
solution_candidates(1:ns)=deal(output); %Form a struct array in the form of the solution struct

%Find solution
while ~isempty(SCS) %Continue until the set of candidate solutions is empty
    counter=counter+1; %Keep track of the number of solutions obtained

    if counter>ns
        %Iteration number exceeds the expected number of iterations. Issue a warning.
        warning('Iteration number: %d! The expected number of iterations %d has been exceeded.', counter, ns);
        announce(sprintf('Iteration number: %d! The expected number of iterations %d has been exceeded.', counter, ns));
    end
    
    solution_candidates(counter)=fcn_handle();
    if ~isempty(SCS)
        announce(sprintf('%s: Iteration %d ended. Number of remaining iterations is at most %d', fcn_name, counter, ns-counter));
    else
        announce(sprintf('%s: Last iteration (%d) completed', fcn_name, counter));
    end
end

%No sample left in the set of candidate solutions at this point. Select the
%most successful solution among the accepted ones
solution_candidates=solution_candidates(1:counter);
solution_table=zeros(counter,2);
for i=1:counter
    solution_table(i,1)=solution_candidates(i).accepted;
    solution_table(i,2)=solution_candidates(i).success;
end
ind=find(solution_table(:,1));%Find the indices of the accepted solutions
[~, ind2]=max(solution_table(ind,2));%Find the index of the most successful of those
ind=ind(ind2);%Find its index in the table

solution=solution_candidates(ind); %Return the solution
%If not empty, share the intermediate solutions too.
if ~isempty(solution)
    solution.solution_candidates=solution_candidates; 
end
