function P=compute_KS_P(data, mu, sigma)
% P=compute_KS_P(data, mu, sigma)
%
% Uses the Kolmogorov-Smirnov test to determine the goodness-of-fit of a normal pdf defined by mu and sigma and that is truncated 
% in the interval [min(data) max(data)] for 'data'
%
% Author: Murat Okatan, PhD.
% Citation: Okatan M, Kocatrk M. Truncation thresholds: a pair of spike detection thresholds computed using truncated probability distributions. 
% Turk J Elec Eng & Comp Sci 2016; Available online: 12.05.2016.
% http://online.journals.tubitak.gov.tr/openAcceptedDocument.htm?fileID=696635&no=172411
% DOI: 10.3906/elk-1603-33 (This number will become active after the manuscript has been selected for inclusion in an issue.)
%
% This work is supported by the Scientific Research Project Fund of Cumhuriyet University under the project number TEKNO-002 
% Copyright (c) 2015 Cumhuriyet University
% Bu alma, Cumhuriyet niversitesi Bilimsel Aratrma Projeleri (CBAP) tarafndan TEKNO-002 proje numaras ile desteklenmitir.
% Telif Hakk (c) 2015 Cumhuriyet niversitesi

low=min(data);
high=max(data);

%Cumulative distribution function
cdfn = (normcdf(sort(unique(data)), mu, sigma)-normcdf(min(data), mu, sigma))/(normcdf(high, mu, sigma) - normcdf(low, mu, sigma));
[~, P]=kstest(data, [sort(unique(data)), cdfn]);
