function [mu, sigma, confidence_interval]=fit_truncated(data)
% [mu, sigma]=fit_truncated(data)
% This function fits the normal distribution function truncated between [min(data) ve max(data)] to 'data' by maximum likelihood and returns the extimated 
% mu and sigma along with their 95% confidence interval
%
% Author: Murat Okatan, PhD.
% Citation: Okatan M, Kocatrk M. Truncation thresholds: a pair of spike detection thresholds computed using truncated probability distributions. 
% Turk J Elec Eng & Comp Sci 2016; Available online: 12.05.2016.
% http://online.journals.tubitak.gov.tr/openAcceptedDocument.htm?fileID=696635&no=172411
% DOI: 10.3906/elk-1603-33 (This number will become active after the manuscript has been selected for inclusion in an issue.)
%
% This work is supported by the Scientific Research Project Fund of Cumhuriyet University under the project number TEKNO-002 
% Copyright (c) 2015 Cumhuriyet University
% Bu alma, Cumhuriyet niversitesi Bilimsel Aratrma Projeleri (CBAP) tarafndan TEKNO-002 proje numaras ile desteklenmitir.
% Telif Hakk (c) 2015 Cumhuriyet niversitesi
%
% Updates since the first version dated February 8, 2016:
% Nov 26, 2017: Lines 32-34 in this version are enclosed inside the try-catch block in lines 31-39 in this version. This way, when mle.m
% fails to converge, the estimates are noted as NaN and the search for valid solution candidates continues.
% Version date: Nov 26, 2017. Update was made possible thanks to data provided by Assoc. Prof. Dr. LKAY ULUSOY.

low=min(data);
high=max(data);

% define the truncated normal probability density function
tnpdf = @(x, mu, sigma) normpdf(x, mu, sigma) ./ (normcdf(high, mu, sigma)-normcdf(low, mu, sigma));
  
%start values
start_values = [mean(data), std(data)];

%estimate the mu and sigma of the truncated normal
try%m 25.11.2017
    [estimates, confidence_interval] = mle(data, 'pdf', tnpdf, 'start', start_values, 'lower', [-Inf 0]);
    mu=estimates(1);
    sigma=estimates(2);
catch%m 25.11.2017
    mu=NaN;%m 25.11.2017
    sigma=NaN;%m 25.11.2017
    confidence_interval=NaN;%m 25.11.2017
end%m 25.11.2017
